import unittest, copy
import onkopus.onkopus_clients


class DrugClassedTestCase(unittest.TestCase):

    def test_get_drug_list(self):
        genome_version = 'hg38'
        data = {"chr7:140753336A>T": {}, "chr17:7673776G>A": {}}

        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        variant_data = onkopus.onkopus_clients.MetaKBClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.CIViCClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.OncoKBClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.AggregatorClient(
            genome_version=genome_version).process_data(variant_data)

        drug_list = onkopus.onkopus_clients.DrugOnClient(genome_version).get_drug_list(list(variant_data.keys()),variant_data)
        print(drug_list)

    def test_get_drug_classes(self):
        genome_version = 'hg38'

        drug_list = ["Vemurafenib", "Cetuximab", "Olaparib"]

        drug_classifications = onkopus.onkopus_clients.DrugOnClient(genome_version).get_drug_classes(drug_list)
        print("Response ", drug_classifications)




import traceback, datetime


def parse_score_results(vcf_lines,
                            json_body, srv_prefix, error_logfile = None):
    """


    :param vcf_lines:
    :param json_body:
    :param srv_prefix:
    :param error_logfile:
    :return:
    """
    for qid, json_obj in json_body.items():
        if json_obj:

            # calculate percentage
            if 'score' in json_obj:
                json_obj['score_percent'] = int(float(json_obj['score']) * 100)
            elif 'Score' in json_obj:
                json_obj['score_percent'] = int(float(json_obj['Score']) * 100)

            # colour = "#00b0d2"
            #colour = "#ce8585"
            #json_obj['score_color'] = colour

            #for k in self.extract_keys:
            #    if k in json_obj:
            #        pass
            #        # annotations.append('{}_{}={}'.format(self.srv_prefix, k, json_body[i][k]))
            try:
                # json_obj.pop('q_id')
                vcf_lines[qid][srv_prefix] = json_obj
            except:
                print("error")
                if error_logfile is not None:
                    cur_dt = datetime.datetime.now()
                    date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                    print(cur_dt, ": error processing variant response: ", qid, ';', traceback.format_exc(),
                          file=error_logfile + str(date_time) + '.log')
    return vcf_lines


import unittest, os
import onkopus as op
import adagenes

class TestCustomVCFAnnotation(unittest.TestCase):

    def test_custom_vcf_annotation(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/somaticMutations.ln50.vcf"
        genome_version="hg19"
        outfile = infile + ".anno.csv"
        bframe = op.read_file(infile)
        bframe.data = adagenes.LiftoverClient(genome_version="hg19").process_data(bframe.data)
        print(bframe.data)
        bframe.data = op.UTAAdapterClient(bframe.genome_version).process_data(bframe.data)
        bframe.data = op.DBNSFPClient(bframe.genome_version).process_data(bframe.data)
        print(bframe.data)
        op.write_file(outfile, bframe)





import os, time
import onkopus.conf.read_config as conf_reader
import datetime
import adagenes as ag
import onkopus as op


def _db_request(pid, module=None, genome_version=None):
    """

    :param pid:
    :param module:
    :param genome_version:
    :return:
    """
    print("Annotate biomarkers of ID: ", pid, ", genome version: ", genome_version, ", module: ", module)

    onkopus_server_url = conf_reader.__MODULE_PROTOCOL__ + "://" + conf_reader.__MODULE_SERVER__ + conf_reader.__PORT_ONKOPUS_SERVER__
    onkopus_server_url = onkopus_server_url + "/onkopus-server/v1/analyze_variants"

    op.import_data.annotate_variant_data(pid, onkopus_server_url=onkopus_server_url,
                                              genome_version=genome_version, module=module)


def _module_request(input_file, output_file, module, genome_version=None,
                    input_format=None, output_format=None, target=None, data_type="g"):
    """
    Annotates a variant file with the defined module and writes the annotated variant file in the file system.
    Directly employs the Onkopus clients, without writing the results in the Onkopus database

    :param input_file:
    :param output_file:
    :param module:
    :param genome_version:
    :return:
    """
    modules = module.split(",")

    print("Input file ", input_file)

    # Employ CCS GeneToGenomic service if data is in protein format
    # if bframe.data_type == "p":
    #    print("Proteomic data detected: Retrieving genomic locations")
    #    genome_version = "hg38"
    #    client = CCSGeneToGenomicClient(genome_version)
    #    bframe.data = client.process_data(bframe.data, input_format='tsv')

    if (genome_version == "hg19") or (genome_version == "t2t"):
        if module != "liftover":
            print("Annotating variant data to GRCh38...", genome_version)
            output_file_liftover = input_file + ".GRCh38.vcf "
            obj = ag.LiftoverAnnotationClient(genome_version=genome_version, target_genome="hg38")
            print("lo annotation ",input_file,input_format,output_format)
            input_format = None
            output_format = None
            #ag.process_file(input_file, output_file_liftover, obj, genome_version=genome_version,
            #                input_format=input_format,
            #                output_format=output_format, error_logfile=None)
            #ag.process_file(input_file, output_file_liftover, ag.LiftoverAnnotationClient("hg19", target_genome="hg38"))
            ag.process_file(input_file, output_file_liftover, ag.LiftoverClient("hg19", target_genome="hg38"))
            input_file = output_file_liftover
            genome_version = "hg38"
            print("Liftover successful. ",input_file)

    # Annotate
    for m in modules:
        obj = op.get_onkopus_client(m, genome_version, target=target, data_type=data_type)
        # print("client ", type(client))
        if obj is None:
            print("Error: Onkopus module not found: ", m)
            exit(1)#

        #ag.process_file(input_file, output_file, obj, genome_version=genome_version, input_format=input_format,
        #                output_format=output_format, error_logfile=None)
        ag.process_file(input_file, output_file, obj)


def run_interpretation(
                       input_file,
                       output_file,
                       module=None,
                       genome_version=None,
                       pid=None,
                       input_format=None,
                       output_format=None,
                       mode=None,
                       target=None
                       ):
    """
    Annotates an input file with the defined module. The annotated file is saved in the given output path

    :param input_file:
    :param output_file:
    :param module:
    :param genome_version:
    :return:
    """
    if mode == "test":
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        input_file = "somaticMutations.ln50.vcf"
        genome_version = "hg19"
        input_file = __location__ + '/../../tests/test_files' + '/' + input_file

    # print("Starting interpretation")
    time_start = time.time()

    # Check input file parameter
    if input_file == '':
        if pid is None:
            print \
                ("Error: No input file passed. Please define an input file with the -i option, e.g. onkopus run -i /path/to/file/mutations.vcf.gz")
            exit(1)
        else:
            print("Annotate patient biomarkers: " ,pid)
            _db_request(pid, module=module, genome_version=genome_version)
            exit(0)

    file_name, file_extension = os.path.splitext(input_file)
    input_format_recognized = file_extension.lstrip(".")
    if input_format_recognized == "gz":
        # print("Found .gz file")
        file_name, file_extension = os.path.splitext(file_name)
        input_format_recognized = file_extension.lstrip(".")

    if input_format == "":
        input_format = input_format_recognized
        print("Recognized input format: " ,input_format)

    # get output format
    if output_format == "":
        output_format = input_format_recognized
        print("No output format given. Using input file format: " ,input_format)

    if output_file == '':
        datetime_str = str(datetime.datetime.now())
        basefile = os.path.splitext(input_file)[0]
        output_file = basefile + '.' + datetime_str + ".ann." + output_format
        print("No output file defined. Generated output file path: ", output_file)

    if conf_reader.config["DEFAULT"]["MODE"] == "LOC":
        print("local mode")

        # Check if Onkopus modules are running

    elif conf_reader.config["DEFAULT"]["MODE"] == "PUB":
        # print("public mode")

        module_server = conf_reader.__MODULE_PROTOCOL__ + '//' + conf_reader.__MODULE_SERVER__
        print("Module server: ", module_server)


    # Get active modules
    active_modules = []
    try:
        active_modules = conf_reader.config["DEFAULT"]["ACTIVE_MODULES"].split(",")
        print("Active modules: " ,active_modules)

    except:
        print("Error: Could not read active modules")
        exit(1)

    # Query all modules if module is set to 'all'
    # if module == 'all':
    #    for module in active_modules:
    #        self.module_request(input_file, output_file, module, genome_version=genome_version)
    # else:

    _module_request(input_file, output_file, module, genome_version=genome_version,
                         input_format=input_format ,output_format=output_format, target=target)

    print("File annotated: " ,module ,", annotations written in: " ,output_file)

    time_stop = time.time()
    time_total = (time_stop - time_start)
    print("Annotation time: " ,str(time_total))




from onkopus.conf import read_config as config
import copy, traceback
import adagenes as ag
from adagenes.tools.preprocessing import generate_biomarker_frame_from_gene_name_str
from adagenes.tools import generate_keys
from onkopus.processing import annotate_variant_data
from adagenes.tools.format_requests import separate_unidentified_snvs
from adagenes.tools.data_structures import merge_dictionaries
import onkopus


def annotate_snvs(annotated_data, genome_version="hg38", oncokb_key=None,
                  lo_hg19=None,lo_hg38=None,lo_t2t=None,
                  tumor_type=None, request_id=None):
    """

    :param annotated_data:
    :param genome_version:
    :param oncokb_key:
    :param lo_hg19:
    :param lo_hg38:
    :param lo_t2t:
    :param tumor_type:
    :return:
    """
    snv_data = {}
    for var in annotated_data.keys():
        mut_type = ""
        if "variant_data" in annotated_data[var]:
            if "mutation_type" in annotated_data[var]["variant_data"]:
                mut_type = annotated_data[var]["variant_data"]["mutation_type"]
        if "mutation_type" in annotated_data[var]:
            mut_type = annotated_data[var]["mutation_type"]
        #print("mut ",mut_type, ": ",annotated_data[var])
        if mut_type == "snv":
            snv_data[var] = annotated_data[var]

    if len(list(snv_data.keys())) > 0:
        snv_data = annotate_variant_data(snv_data, genome_version=genome_version, oncokb_key=oncokb_key,lo_hg19=lo_hg19,
                                         lo_hg38=lo_hg38,lo_t2t=lo_t2t, tumor_type=tumor_type, request_id=request_id)
    #snv_data = onkopus.onkopus_clients.AggregatorClient(genome_version=genome_version).process_data(snv_data)

    for var in snv_data.keys():
        annotated_data[var] = snv_data[var]

    return annotated_data


def analyze_genomic_location_request(genompos, genome_version: str = 'hg38', data=None, tumor_type=None):
    """

    :param genompos:
    :param genome_version:
    :param data:
    :return:
    """
    print("analyze genomic location request: ",genompos)

    if data is None:
        annotated_data = {}
    else:
        annotated_data = data

    perform_query = False
    if genompos is not None:
        if genompos != '':
            # Check if the input string has the required format
            # TODO
            print("gpos ",genompos)

            # parse genome location
            variant_str = genompos.split(",")
            for var in variant_str:
                try:
                    if var != "":
                        var_form = var.replace("CHR", "chr")
                        annotated_data[var_form] = {config.__FEATURE_QID__: var_form}


                        gene_data = {}
                        #for key in annotated_data.keys():
                        #    gene, variant_exchange = annotated_data[key][config.uta_adapter_srv_prefix]["gene_name"], \
                        #                             annotated_data[key][config.uta_adapter_srv_prefix][
                        #                                 "variant_exchange"]
                        #    gene_data[gene] = {}
                        #    gene_data[gene][config.uta_adapter_srv_prefix] = {
                        #        config.__FEATURE_VARIANT__: variant_exchange,
                        #        config.__FEATURE_GENE__: gene
                        #    }

                        #client = adagenes.onkopus_clients.CCSGeneToGenomicClient(
                        #    genome_version=genome_version)
                        #gene_data_annotated = client.process_data(gene_data)
                        #gene_data_annotated = client.generate_genome_locations_as_keys(gene_data_annotated)

                        #gene_data_annotated_upper = {}
                        #for key in gene_data_annotated.keys():
                        #    newkey = key.upper()
                        #    newkey = newkey.replace("CHR", "chr")
                        # #   gene_data_annotated_upper[newkey] = gene_data_annotated[key]
                        #for key in annotated_data.keys():
                        #    annotated_data[key][config.uta_adapter_genetogenomic_srv_prefix] = \
                        #        gene_data_annotated_upper[key][adagenes.config.uta_adapter_genetogenomic_srv_prefix]
                        #    annotated_data[key][config.variant_data_key] = \
                        #        annotated_data[key][config.variant_data_key] | gene_data_annotated_upper[key][
                        #            adagenes.config.variant_data_key]
                except:
                    print("error parsing genomic positions")
                    print(traceback.format_exc())
                perform_query = True

        if (len(annotated_data) > 0) and perform_query:

            #annotated_data = generate_keys(annotated_data)
            #annotated_data = adagenes.tools.reference_genomes.transform_hg19_in_hg38_bframe(annotated_data,genome_version)
            if genome_version != "hg38":
                print("Run liftover from ",genome_version," to hg38")
                annotated_data = ag.LiftoverClient(genome_version=genome_version).process_data(annotated_data, target_genome="hg38")
                print("after li ",annotated_data)
            genome_version = "hg38"
            annotated_data = ag.clients.LiftoverAnnotationClient(
                genome_version=genome_version).process_data(annotated_data)

            annotated_data = onkopus.onkopus_clients.UTAAdapterClient(
                genome_version=genome_version).process_data(annotated_data)

            annotated_data = annotate_variant_data(annotated_data, genome_version=genome_version, tumor_type=tumor_type)

        for var in annotated_data.keys():
            if "variant_data" in annotated_data[var]:
                annotated_data[var]["variant_data"]["genomic_location"] = var

        return annotated_data


def analyze_snv_request(
                            gene_names_prot_change=None,
                            genome_version: str = 'hg38',
                            data=None,
                            oncokb_key='',
                            lo_hg19=None,
                            lo_hg38=None,
                            lo_t2t=None,
                            tumor_type=None
                        ):
    """
    Annotates variants identified by gene name and protein change

    :param gene_names_prot_change: Comma-separated list of genes and protein change ('BRAF:V600E,TP53:R282W')
    :param genome_version:
    :param data
    :return:
    """
    print("analyze variant search: ", gene_names_prot_change)
    if gene_names_prot_change == "":
        return {}, {}

    if data is None:
        annotated_data = {}
    else:
        annotated_data = data

    perform_query = False
    unidentified_snvs={}

    # retrieve genomic data if gene name and protein change are given
    if (gene_names_prot_change is not None) and (gene_names_prot_change != ''):
        gene_data = generate_biomarker_frame_from_gene_name_str(gene_names_prot_change)

        client = onkopus.onkopus_clients.CCSGeneToGenomicClient(genome_version=genome_version)
        annotated_data = client.process_data(copy.deepcopy(gene_data), input_format='json')

        annotated_data = separate_unidentified_snvs(annotated_data)

        annotated_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(annotated_data)

        perform_query = True

    if (len(annotated_data) > 0) and perform_query:
        annotated_data = generate_keys(annotated_data,config.onkopus_modules)
        #print("analyze variant data request")
        #print("gv",genome_version," keys ", list(annotated_data.keys()))
        #annotated_data = ag.tools.reference_genomes.transform_hg19_in_hg38_bframe(annotated_data, genome_version)

        if genome_version != "hg38":
            #annotated_data = ag.clients.LiftoverClient(
            #    genome_version=genome_version).process_data(annotated_data, lo_hg19=lo_hg19, lo_hg38=lo_hg38)
            # print("Liftover: ",annotated_data)
            #annotated_data = generate_keys(annotated_data)
            annotated_data = ag.LiftoverClient(genome_version=genome_version).process_data(annotated_data,
                                                                                                 target_genome="hg38")
        genome_version = "hg38"

        if genome_version == "hg38":
            annotated_data = ag.tools.reference_genomes.add_hg38_positions(annotated_data)
        genome_version = "hg38"
        #print("keys ", list(annotated_data.keys()))
        annotated_data = annotate_variant_data(annotated_data, genome_version=genome_version, oncokb_key=oncokb_key,
                                               lo_hg19=lo_hg19,lo_hg38=lo_hg38,lo_t2t=lo_t2t,
                                               tumor_type=tumor_type)

    for var in annotated_data.keys():
        if "variant_data" in annotated_data[var]:
            annotated_data[var]["variant_data"]["genomic_location"] = var

    #annotated_data = merge_dictionaries(annotated_data,unidentified_snvs)

    return annotated_data



import unittest
import adagenes as ag
import onkopus.onkopus_clients


class UTAAdapterGeneAnnotationTestCase(unittest.TestCase):

    def test_uta_adapter_genetogenomic_client(self):
        genome_version = 'hg38'
        data = {"NRAS:Q61L": {}, "TP53:R282W": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(bframe.data)
        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(variant_data)
        print("Response ",variant_data["chr1:114713908T>A"]["UTA_Adapter"])
        qids = ["chr1:114713908T>A", "chr17:7673776G>A"]
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter GeneToGenomic")
        self.assertEqual(variant_data["chr1:114713908T>A"]["mutation_type"],"snv","")

    def test_uta_adapter_genetogenomic_geneannotation(self):
        genome_version = 'hg38'
        data = {"chr18:7888143G>A": {}, "chr10:87864472G>A": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(bframe.data)
        variant_data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(variant_data)
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version, data_type="g").process_data(variant_data)
        qids = ["chr18:7888143G>A", "chr10:87864472G>A"]
        self.assertListEqual(list(variant_data.keys()), qids, "Error UTA adapter GeneToGenomic")
        self.assertEqual(variant_data["chr18:7888143G>A"]["UTA_Adapter"]["variant_exchange"], "E78=", "")



import unittest, copy
import onkopus.onkopus_clients

class VUSPredictAnnotationTestCase(unittest.TestCase):

    def test_vus_predict_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}

        data = onkopus.onkopus_clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)

        variant_data = onkopus.onkopus_clients.VUSPredictClient(
            genome_version=genome_version).process_data(data)

        print("Response ",variant_data)





import unittest, copy
import onkopus.onkopus_clients
import adagenes


class BLOSUMTestCase(unittest.TestCase):

    def test_blosum_client(self):
        genome_version = 'hg38'
        data = {"NRAS:Q61L": {}, "TP53:R282W": {}, "MUTYH:L420M": {}}
        variant_data = onkopus.onkopus_clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(data)
        variant_data = adagenes.BLOSUMClient().process_data(variant_data)


        print("Response ",variant_data)




import adagenes
import onkopus.onkopus_clients


def annotate_indels(annotated_data, genome_version="hg38",
                    oncokb_key=None, tumor_type=None,
                    request_id=None):

    snv_data = {}
    for var in annotated_data.keys():
        mut_type = ""
        if "variant_data" in annotated_data[var]:
            if "mutation_type" in annotated_data[var]["variant_data"]:
                mut_type = annotated_data[var]["variant_data"]["mutation_type"]
        elif "mutation_type" in annotated_data[var]:
            mut_type = annotated_data[var]["mutation_type"]
        if mut_type == "indel":
            snv_data[var] = annotated_data[var]

    if len(list(snv_data.keys())) > 0:
        snv_data = annotate_indel_data(snv_data, genome_version=genome_version, oncokb_key=oncokb_key, tumor_type=tumor_type)
        snv_data = onkopus.onkopus_clients.AggregatorClient(genome_version=genome_version).process_data(snv_data)

    #for var in snv_data.keys():
    #    annotated_data[var] = snv_data[var]
    #return annotated_data
    return snv_data


def indel_request(indels, genome_version="hg38", tumor_type=None):
    """

    :param indel_str:
    :param genome_version:
    :return:
    """
    if len(indels) == 0:
        return {}
    annotated_data = {}
    if len(indels) > 0:
        for indel in indels:
            annotated_data[indel] = {}

            annotated_data[indel] = adagenes.generate_variant_data_section(annotated_data[indel],qid=indel)

    annotated_data = annotate_indel_data(annotated_data, genome_version=genome_version, tumor_type=tumor_type)
    annotated_data = adagenes.tools.get_biomarker_type_aaexchange(annotated_data)

    return annotated_data


def annotate_indel_data(annotated_data, genome_version="hg38", oncokb_key=None, tumor_type=None):
    """
    Annotate insertions and deletions

    :param annotated_data:
    :param genome_version:
    :param oncokb_key:
    :param tumor_type:
    :return:
    """
    annotated_data = onkopus.GENCODEGenomicClient(genome_version=genome_version).process_data(annotated_data)
    annotated_data = onkopus.ClinVarClient(genome_version=genome_version).process_data(annotated_data)
    annotated_data = onkopus.IndelToGeneClient(genome_version=genome_version).process_data(annotated_data)

    return annotated_data




import datetime, traceback, copy
import adagenes.tools.parse_vcf
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as conf_reader

qid_key = "q_id"
error_logfile=None


class MVPClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = conf_reader.mvp_info_lines
        self.url_pattern = conf_reader.mvp_src
        self.srv_prefix = conf_reader.mvp_srv_prefix
        self.extract_keys = conf_reader.mvp_keys

        self.qid_key = "q_id"
        #if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
        #    self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def get_qids_hg38(self,qids_partial,vcf_lines):
        qids_refgen = []

        for qid in qids_partial:
            if "POS_hg19" in vcf_lines[qid][conf_reader.variant_data_key]:
                pos_hg38 = vcf_lines[qid][conf_reader.variant_data_key]["POS_hg19"]
                qid_hg38 = "chr" + vcf_lines[qid][conf_reader.variant_data_key]["CHROM"]+":"+ str(pos_hg38) + \
                    vcf_lines[qid][conf_reader.variant_data_key]["REF"] + ">" + \
                    vcf_lines[qid][conf_reader.variant_data_key]["ALT"]
                qids_refgen.append(qid_hg38)
            else:
                print("No hg19 position found: ",qid)
                qids_refgen.append("")

        return qids_refgen

    def process_data(self, vcf_lines, input_format='json'):

        if (self.genome_version == 'hg38') or (self.genome_version == 'GRCh38'):
            if input_format == 'vcf':
                keys = [conf_reader.uta_adapter_liftover_srv_prefix + "_" + "liftover_position"]
                annotations = adagenes.tools.parse_vcf.extract_annotations_vcf(vcf_lines, keys)
            else:
                keys = ['POS_hg19']
                annotations = adagenes.tools.parse_vcf.extract_annotations_json(vcf_lines, conf_reader.variant_data_key, keys)

            print("extracted positions: ",annotations)
            positions_hg19 = annotations[ keys[0] ]
            positions = dict(zip(annotations[keys[0]], list(vcf_lines.keys())))
            q_ids = annotations[ 'q_id' ]
            try :
                query = ",".join(positions_hg19)
            except:
                print("Error generating MVP query positions")
                print(traceback.format_exc())
                return vcf_lines
        else:
            query = ",".join(list(vcf_lines.keys()))
            list(vcf_lines.keys())
            q_ids = list(vcf_lines.keys())
            positions = None

        qid_list = copy.deepcopy(list(vcf_lines.keys()))

        while True:
            max_length = int(conf_reader.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)

            qids_partial = qid_list[0:max_length]


            try:
                if (self.genome_version == 'hg38') or (self.genome_version == 'GRCh38'):
                    qids_refgen = self.get_qids_hg38(qids_partial, vcf_lines)
                else:
                    qids_refgen = qids_partial

                variants = ','.join(adagenes.tools.filter_alternate_alleles(qids_refgen))
            except:
                print("Error: Could not retrieve liftover positions")
                print(traceback.format_exc())
                return vcf_lines

            try:
                json_body = req.get_connection(variants, self.url_pattern, 'hg19')

                for i, key in enumerate(json_body.keys()):

                    if "Score" in json_body[key]:
                        # calculate percentage of MVP score
                        json_body[key]['score_percent'] = int(json_body[key]["Score"] * 100)

                        if (self.genome_version=='hg38') or (self.genome_version == 'GRCh38'):
                            pos_hg19 = qids_partial[i]
                            vcf_lines[pos_hg19][self.srv_prefix] = json_body[key]
                        else:
                            #vcf_lines[q_id][self.srv_prefix] = json_body[positions[i]]
                            vcf_lines[key][self.srv_prefix] = json_body[key]

            except:
                if self.error_logfile:
                    print("error processing request: ", query, file=self.error_logfile + str(datetime.date_time) + '.log')
                else:
                    print(": error processing variant response: ;", traceback.format_exc())

            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break

        return vcf_lines



import unittest, copy
import onkopus
"""
class AnnotationTestCase(unittest.TestCase):

    def test_oncokb_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        variant_dc = {"0": "chr17:7681744T>C", "1": "chr10:8115913C>T"}

        variant_data = onkopus.clients.OncoKBClient(
            genome_version=genome_version).process_data(data)

        print("OncoKB response ",variant_data)

    def test_ccs_liftover_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        variant_dc = {"0": "chr17:7681744T>C", "1": "chr10:8115913C>T"}

        data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)

        variant_data = onkopus.clients.LiftOverClient(
            genome_version=genome_version).process_data(data)

        print("LiftOver Response: ", variant_data)

    def test_civic_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr17:7673776G>A": {}}
        variant_dc = {"0": "chr7:140753336A>T", "1": "chr17:7673776G>A"}

        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta res", variant_data)

        client = onkopus.clients.CIViCClient(genome_version=genome_version)
        res = client.process_data(variant_data)

        print("CIViC response ",res)

    def test_mvp_client_hg38(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr17:7673776G>A": {}}
        variant_dc = {"0": "chr7:140753336A>T", "1": "chr17:7673776G>A"}

        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)

        variant_data = onkopus.clients.LiftOverClient(
            genome_version=genome_version).process_data(variant_data)

        print("uta res", variant_data)

        client = onkopus.clients.MVPClient(genome_version=genome_version)
        client.process_data(variant_data)

    def test_mvp_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        variant_dc = {"0": "chr17:7681744T>C", "1": "chr10:8115913C>T"}

        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)

        client = onkopus.clients.MVPClient(genome_version=genome_version)
        client.process_data(variant_data)

    def test_primateai_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        variant_dc = {"0": "chr17:7681744T>C", "1": "chr10:8115913C>T"}

        variant_data = onkopus.clients.PrimateAIClient(
            genome_version=genome_version).process_data(data)
        print(variant_data)

    def test_dbnsfp_client(self):
        genome_version = 'hg38'

        data = {"chr14:67885931T>G": {}, "chr7:140753336A>T": {}}
        variant_dc = {"0": "chr14:67885931T>G", "1": "chr7:140753336A>T"}

        variant_data = onkopus.clients.DBSNPClient(
            genome_version=genome_version).process_data(data)
        print(variant_data)

    def test_uta_adapter_genomic_client(self):
        genome_version = 'hg19'
        data = {"TP53:R282W": { "UTA_Adapter": {"gene_name":"TP53", "variant_exchange":"R282W"} }}
        variant_data = onkopus.clients.CCSGeneToGenomicClient(
            genome_version=genome_version).process_data(data)
        print("UTA response ", variant_data)

    def test_uta_adapter_client(self):
        genome_version = 'hg19'
        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        variant_dc = {"0": "chr17:7681744T>C", "1": "chr10:8115913C>T"}
        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("UTA response ",variant_data)

    def test_vuspredict_client(self):
        genome_version='hg19'

        data = { "chr17:7681744T>C" : {  }, "chr10:8115913C>T":{} }
        variant_dc = { "0": "chr17:7681744T>C", "1": "chr10:8115913C>T" }

        variant_data = onkopus.clients.UTAAdapterClient(genome_version=genome_version).process_data(data)

        client = onkopus.clients.VUSPredictClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)

        print("VUS-Predict annotation ",variant_data)

    def test_loftool_client(self):
        genome_version='hg19'

        data = { "chr17:7681744T>C" : {  }, "chr10:8115913C>T":{} }
        variant_dc = { "0": "chr17:7681744T>C", "1": "chr10:8115913C>T" }

        variant_data = onkopus.clients.UTAAdapterClient(genome_version=genome_version).process_data(data)
        print("uta res",variant_data)

        client = onkopus.clients.LoFToolClient(genome_version=genome_version)
        client.process_data(variant_data)

    def test_metakb_client(self):
        genome_version = 'hg38'

        data = { "chr7:140753336A>T": {}, "chr12:25245350C>T":{} }
        variant_dc = {"0": "chr7:140753336A>T", "1": "chr12:25245350C>T" }

        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = onkopus.clients.MetaKBClient(genome_version=genome_version)
        client.process_data(variant_data)

    def test_metakb_client_gene_only(self):
        genome_version = 'hg38'

        data = { "chr7:140753336A>T": {}, "chr12:25245350C>T":{} }
        variant_dc = {"0": "chr7:140753336A>T", "1": "chr12:25245350C>T" }

        variant_data = onkopus.clients.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        keys = copy.deepcopy(list(variant_data.keys()))
        for key in keys:
            variant_data[key]['UTA_Adapter'].pop('variant_exchange')
        print("uta response", variant_data)

        client = onkopus.clients.MetaKBClient(genome_version=genome_version)
        client.process_data(variant_data)
"""


import unittest, copy
import onkopus as op


class DGIDBAnnotationTestCase(unittest.TestCase):

    def test_dgidb_gene_request(self):
        genome_version="hg38"
        data = {"BRAF":{}}
        data = op.DGIdbClient(genome_version=genome_version).process_data(data, gene_request=True)
        self.assertEqual(data["BRAF"]["dgidb"]["summary"][:20], "SORAFENIB:inhibitor:", "")

    def test_dgidb_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr1:2556710C>A": {},
                "chr1:2556710C>T":{},"chr1:2556714A>G":{}, "chr1:2556718C>T":{}}

        variant_data = op.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        variant_data = op.DGIdbClient(genome_version=genome_version).process_data(variant_data)

        print("Response ",variant_data["chr7:140753336A>T"]["dgidb"]["summary"])
        self.assertEqual(variant_data["chr7:140753336A>T"]["dgidb"]["summary"][:20],"SORAFENIB:inhibitor:","")



import unittest
import onkopus


class AggregationTestCase(unittest.TestCase):

    def test_aggregator_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr12:25245350C>T": {}}

        variant_data = onkopus.onkopus_clients.uta_adapter_client.UTAAdapterClient(
            genome_version=genome_version).process_data(data)
        print("uta response", variant_data)

        client = onkopus.onkopus_clients.MetaKBClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.CIViCClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)
        client = onkopus.onkopus_clients.OncoKBClient(genome_version=genome_version)
        variant_data = client.process_data(variant_data)

        # Aggregator
        variant_data = onkopus.onkopus_clients.aggregator_client.AggregatorClient(
            genome_version=genome_version).process_data(variant_data)

        #print("Aggregator response ",variant_data)
        self.assertEqual(len(variant_data["chr7:140753336A>T"]["onkopus_aggregator"]["merged_match_types_data"]), 701, "")
        self.assertListEqual(list(variant_data["chr7:140753336A>T"].keys()),
                             ['variant_data', 'UTA_Adapter', 'metakb', 'civic', 'onkopus_aggregator'], "")



from threading import Thread


class ThreadWithReturnValue(Thread):

    def __init__(self, group=None, target=None, name=None,
                 args=(), kwargs={}, Verbose=None):
        Thread.__init__(self, group, target, name, args, kwargs)
        self._return = None

    def run(self):
        if self._target is not None:
            self._return = self._target(*self._args,
                                        **self._kwargs)

    def join(self, *args):
        Thread.join(self, *args)
        return self._return



import unittest, os
import onkopus


class TestCLIAnnotation(unittest.TestCase):
#

    def test_cli_annotation(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.hg38.ln200.vcf"
        outfile = infile + ".anno.csv"
        bframe = onkopus.read_file(infile)
        bframe.data = onkopus.annotate_variant_data(bframe.data, genome_version="hg19")
        onkopus.write_file(outfile,bframe)

        for var in bframe.data.keys():
            vartype = bframe.data[var]["type"]
            #print(vartype)
            #print(var, bframe.data[var])

        self.assertEqual(bframe.data["chr7:21745009A>G"]["UTA_Adapter_gene"]["hgnc_symbol"],"DNAH11","")




import os, traceback, copy, gzip
import pandas as pd
import adagenes
import adagenes.conf.read_config as conf_reader


class ClinSigWriter():
    """
    Export treatment data on the clinical significance of potential treatments
    """

    def export_feature_results_lv2(self, biomarker_data, record_path=None, meta=None) -> pd.DataFrame:
        """
        Exports biomarker data in full mode with level 2 meta data

        :param biomarker_data:
        :param outfile_src:
        :param feature:
        :param record_path:
        :param meta:
        :param sep:
        :return:
        """
        df_sum = pd.DataFrame()
        for var in biomarker_data.keys():
            if "UTA_Adapter" in biomarker_data[var].keys():
                try:
                    df = pd.json_normalize(data=biomarker_data[var], record_path=record_path, meta=meta, errors='ignore')
                    df_sum = pd.concat([df_sum, df], axis=0)
                except:
                    print(traceback.format_exc())
        return df_sum

    def fill_in_missing_keys_lv2(self,biomarker_data, struc):

        for var in biomarker_data.keys():
            for key in struc:
                if key not in biomarker_data[var]:
                    biomarker_data[var][key] = {}
                for val in struc[key]:
                    if val not in biomarker_data[var][key]:
                        biomarker_data[var][key][val] = {}

        return biomarker_data

    def write_evidence_data_to_file_all_features(self,bframe,
                                                 databases=None,
                                                 format='tsv',
                                                 sep='\t',
                                                 outfile=None):
        """
        Writes data on clinical significance on treatments on molecular targets in an output file,
        where each row represents a potential treatment.

        :param variant_data:
        :param databases:
        :param output_file:
        :param format:
        :param sep:
        :return:
        """

        variant_data = bframe.data

        for var in variant_data.keys():
            if "onkopus_aggregator" not in variant_data[var]:
                variant_data[var]["onkopus_aggregator"] = {}
            if "merged_match_types_data" not in variant_data[var]["onkopus_aggregator"]:
                variant_data[var]["onkopus_aggregator"]["merged_match_types_data"] = []

            if "UTA_Adapter" not in variant_data[var]:
                variant_data[var]["UTA_Adapter"] = {}
            if "gene_name" not in variant_data[var]["UTA_Adapter"]:
                variant_data[var]["UTA_Adapter"]["gene_name"] = ""
            if "variant_exchange" not in variant_data[var]["UTA_Adapter"]:
                variant_data[var]["UTA_Adapter"]["variant_exchange"] = ""

        #record_path = [
        #    ["onkopus_aggregator", "merged_match_types_data",
        #        "exact_match"]]
        record_path = [["onkopus_aggregator", "merged_match_types_data"]]
        meta = [["UTA_Adapter", "gene_name"], ["UTA_Adapter", "variant_exchange"]]

        df = self.export_feature_results_lv2(copy.copy(variant_data), record_path=record_path, meta=meta)
        #print(df.columns)
        #print(df["drugs"])

        #for column in df.columns:
        #    df[column] = df[column].replace("\"","")
        #    df[column] = df[column].replace("\n", "")
        #    df[column] = df[column].replace("'", "")
        #    df[column] = df[column].replace("\t", " ")
        #    df[column] = df[column].replace(",", "")

        df = self.normalize_values(df)

        def remove_brackets_and_quotes(value):
            if isinstance(value, str) and value.startswith("['") and value.endswith("']"):
                return value[2:-2]
            if '"' in str(value):
                value = value.replace('"', '')
            return value

        # Apply the function to each cell in the DataFrame
        df = df.applymap(remove_brackets_and_quotes)

        header = sep.join(df.columns)
        outfile.write(header + '\n')

        #for _, row in df.iterrows():
        #    #line = sep.join(map(str, row.values))  # Convert each value to string and join with commas
        #    row_data = [str(value) for value in row]
        #    print(','.join(row_data) + '\n')
        #    output_file.write(','.join(row_data) + '\n')
        #df.to_csv(output_file)

        #df.to_csv(outfile)

        for i in range(0, df.shape[0]):
            fields = []
            not_empty = False
            not_nans = False
            for j in range(df.shape[1]):
                fields.append(str(df.iloc[i,j]))
                if (j>0) and (str(df.iloc[i,j]) != "") and (str(df.iloc[i,j]) != 'nan'):
                    not_empty = True
                    not_nans = True
            #line = sep.join(fields)
            line = sep.join(f'"{field}"' for field in fields)
            if (line != "") and (not_empty is True) and (not_nans is True):
                #print("line: ",line, "fields: ",fields)
                outfile.write(line + '\n')


    def normalize_values(self,df):
        """
        Formats drug names and drug classifications for treatment export

        :param df:
        :return:
        """
        drug_names = []
        drug_classes = []
        for i in range(0,df.shape[0]):
            drug_str = ""
            dc_str = ""
            if "drugs" in df.columns:
                for drug in df.iloc[i,:].loc["drugs"]:
                    if "drug_name" in drug:
                        drug_name = str(drug["drug_name"])
                        drug_str += drug_name + ", "
                    if isinstance(drug, dict):
                        if "drug_class" in drug.keys():
                            if isinstance(drug["drug_class"], str):
                                dc_str += str(drug["drug_class"]) + ','
                            elif isinstance(drug["drug_class"], list):
                                for dclass in drug["drug_class"]:
                                    dc_str += str(dclass) + ','

                drug_str = drug_str.rstrip(", ")
            #df.at[i,"drugs"] = drug_str

            dc_str = dc_str.rstrip(',')
            drug_names.append(drug_str)
            drug_classes.append(dc_str)
        #print("def shape ",df.shape[0],":", df.shape," len classes ",len(drug_classes))
        try:
            df.drop(["drugs"], axis=1)
        except:
            print(traceback.format_exc())

        df["drug_class"] = drug_classes
        df["drugs"] = drug_names

        return df

    def write_evidence_data_to_file(self,variant_data,outfile,sep="\t"):
        """

        :param outfile:
        :param variant_data:
        :param sep:
        :return:
        """

        fileopen = False
        if isinstance(outfile, str):
            fileopen = True
            file_name, file_extension = copy.deepcopy(os.path.splitext(outfile))
            input_format_recognized = file_extension.lstrip(".")
            if input_format_recognized == "gz":
                outfile = gzip.open(outfile, 'wt')
            else:
                outfile = open(outfile, "w", encoding='utf-8')

        if isinstance(variant_data, dict):
            bframe = adagenes.BiomarkerFrame(data=variant_data)
        else:
            bframe = variant_data
        self.write_evidence_data_to_file_all_features(bframe,sep=sep,outfile=outfile)

        if fileopen is True:
            outfile.close()
        outfile.close()

    def write_evidence_data_to_file2(self,variant_data, databases=None,output_file=None,format='tsv', sep='\t'):

        if isinstance(variant_data,adagenes.BiomarkerFrame):
            variant_data = variant_data.data

        if databases is None:
            databases = conf_reader.config["DEFAULT"]["ACTIVE_EVIDENCE_DATABASES"].split()

        if format == 'csv':
            sep=','

        if output_file is None:
            print("not output file given")
            return

        #print(output_file)
        outfile = open(output_file, 'w')
        line = 'biomarker' + '\t' + 'disease' + '\t' + 'drugs' + '\t' + 'drug_class' + '\t' + 'evidence_level' + '\t' + 'citation_id' + '\t' + 'source'
        print(line, file=outfile)

        for var in variant_data.keys():
            #print(variant_data[var].keys())

            #for db in databases:
            if 'onkopus_aggregator' in variant_data[var]:
                    if 'merged_match_types_data' in variant_data[var]['onkopus_aggregator']:
                            print(len(variant_data[var]['onkopus_aggregator']['merged_match_types_data']))
                            for result in variant_data[var]['onkopus_aggregator']['merged_match_types_data']:
                                #print(result)
                                drugs = result['drugs']
                                drug_name = ""
                                drug_class = ""
                                for drug in drugs:
                                    #print(drug)
                                    if isinstance(drug, dict):
                                        if "drug_name" in drug:
                                            drug_name += drug["drug_name"] + ","
                                        if "drug_class" in drug:
                                            drug_class += drug["drug_clas"] + ','
                                drug_name = drug_name.rstrip(",")
                                line = str(result['biomarker']) + '\t' + str(result['disease']) \
                                       + '\t' + str(drug_name) + '\t' + drug_class + '\t' + str(result['evidence_level']) \
                                       + '\t' + str(result['citation_id']) + '\t' + str(result['source'])
                                print(line, file=outfile)
            else:
                    print("no data: ")

        outfile.close()


    def write_to_file(self, outfile, bframe, databases=None, format="tsv", sep="\t"):
        """

        :param output_file:
        :param bframe:
        :param databases:
        :param format:
        :param sep:
        :return:
        """
        isfile=False
        if isinstance(outfile, str):
            outfile = open(outfile, 'w')
            isffle = True

        self.write_evidence_data_to_file_all_features(bframe, outfile=outfile, format=format,
                                                 sep=sep)

        if isfile is True:
            outfile.close()



import datetime, traceback, copy
import adagenes.tools
from onkopus.conf import read_config as config
import adagenes as ag


class DBSNPClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.dbsnp_src
        self.srv_prefix = config.dbsnp_srv_prefix
        self.extract_keys = config.dbsnp_keys
        self.info_lines = config.dbsnp_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines):
        """
        Annotates biomarker data with data from the dbSNP database

        :param vcf_lines:
        :return:
        """
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)
        retransform = False
        if self.genome_version == "hg38":
            qid_list = copy.deepcopy(list(vcf_linesf.keys()))
            qid_dc = None
        else:
            qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
            self.genome_version = "hg38"
            retransform = True

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]

            q = ','.join(adagenes.tools.filter_alternate_alleles(qids_partial))

            vcf_lines = adagenes.processing.parse_http_responses.parse_module_response(q, vcf_lines, self.url_pattern,
                                                                                     self.genome_version,
                                                                                     self.srv_prefix,
                                                                                        qid_dc,retransform)

            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break

        return vcf_lines



import requests

def add_patient_to_mtb(pid, mtb, onkopus_server_url):
    """

    :param pid:
    :param mtb
    :param onkopus_server_url:
    :return:
    """
    url = onkopus_server_url + "?pid=" + str(pid) + "&mtb_id=" + str(mtb) + "&action=a"
    print("Add patient to MTB: ", url)
    res = requests.get(url)
    print(res)



import adagenes
import onkopus as op
import onkopus.conf.read_config as conf_reader


def divide_list(input_list, chunk_size=100):
    """Divides a list into sublists of specified size."""
    for i in range(0, len(input_list), chunk_size):
        yield input_list[i:i + chunk_size]


def read_file(infile,input_format=None, mapping=None, genome_version=None, sep=","):
    """
    Reads in a variant file of different formats (.vcf, .maf, .csv, .tsv, .xlsx, .txt)

    :param infile:
    :param input_format:
    :param mapping:
    :param genome_version:
    :return:
    """
    bframe = adagenes.read_file(infile, reader=None, input_format=input_format, mapping=mapping, genome_version=genome_version, sep=sep)

    if bframe.data_type == "p":
        bframe.genome_version = "hg38"
        bframe.data = op.CCSGeneToGenomicClient(bframe.genome_version).process_data(bframe.data)
        bframe.data_type="g"

    if bframe.genome_version == "hg19":
        bframe.data = adagenes.LiftoverClient(genome_version=bframe.genome_version).process_data(bframe.data)
        genome_version = "hg38"

    return bframe


def save_variants(outfile, json_obj,file_type=None, genome_version=None, sort_features=True):
    write_file(outfile, json_obj,file_type=None, genome_version=None, sort_features=True)

def save_treatments(outfile, json_obj,file_type=None, genome_version=None, sort_features=True):
    op.ClinSigWriter().write_to_file(outfile, json_obj)

def write_file(outfile, json_obj,file_type=None, genome_version=None, sort_features=True, export_features=None):
    """
    Writes annotated variant data in an output file of the specified output format

    :param outfile:
    :param json_obj:
    :param file_type:
    :param genome_version:
    :param sort_features:
    :return:
    """
    if isinstance(json_obj,dict):
        data = adagenes.BiomarkerFrame()
        data.data = json_obj
        data.genome_version=genome_version

        if export_features is not None:
            mapping = None
        else:
            mapping = conf_reader.tsv_mappings
            if file_type is None:
                writer = adagenes.get_writer(outfile, file_type=file_type)
                if isinstance(writer, adagenes.VCFWriter):
                    mapping = conf_reader.vcf_mappings

        adagenes.write_file(outfile, data,
                          file_type=file_type,
                          genome_version=genome_version,
                          labels=conf_reader.tsv_labels,
                          ranked_labels=conf_reader.tsv_feature_ranking,
                          mapping=mapping,
                            export_features=export_features
                          )
    else:
        if export_features is not None:
            mapping = None
        else:
            mapping = conf_reader.tsv_mappings

        adagenes.write_file(outfile,
                          json_obj,
                          file_type=file_type,
                          genome_version=genome_version,
                          labels=conf_reader.tsv_labels,
                          ranked_labels=conf_reader.tsv_feature_ranking,
                          mapping=mapping,
                            export_features=export_features
                          )




import traceback, datetime, json
import redis
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes as ag


def get_clinev_data(biomarker_data):
    ce_data = {}

    for var in biomarker_data.keys():
        ce_data[var] = {}
        if "civic" in biomarker_data[var].keys():
            if "civic_features_norm" in biomarker_data[var]["civic"].keys():
                ce_data[var]["civic"] = {}
                ce_data[var]["civic"]["civic_features_norm"] =biomarker_data[var]["civic"]["civic_features_norm"]

        if "metakb" in biomarker_data[var].keys():
            if "metakb_features_norm" in biomarker_data[var]["metakb"].keys():
                ce_data[var]["metakb"] = {}
                ce_data[var]["metakb"]["metakb_features_norm"] =biomarker_data[var]["metakb"]["metakb_features_norm"]

        if "oncokb" in biomarker_data[var].keys():
            if "oncokb_features_norm" in biomarker_data[var]["oncokb"].keys():
                ce_data[var]["oncokb"] = {}
                ce_data[var]["oncokb"]["oncokb_features_norm"] =biomarker_data[var]["oncokb"]["oncokb_features_norm"]
    return ce_data


class AggregatorClient:

    def __init__(self, genome_version, request_id=None, tumor_type = None, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= config.onkopus_aggregator_info_lines
        self.url_pattern = config.onkopus_aggregator_src
        self.srv_prefix = config.onkopus_aggregator_srv_prefix
        self.response_keys = config.onkopus_aggregator_response_keys
        self.extract_keys = config.onkopus_aggregator_keys

        self.request_id = request_id
        self.tumor_type = tumor_type

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

        self.redis_client = redis.StrictRedis(host=config.__REDIS_SERVER__, port=config.__REDIS_SERVER_PORT__,
                                         db=config.__REDIS_SERVER_DB__, decode_responses=True)

    def process_data(self, biomarker_data, tumor_type=None):

        #print("request id ",self.request_id)

        if (self.tumor_type is None) and (tumor_type is not None):
            self.tumor_type = tumor_type

        try:
            if self.request_id is None:
                ce_data = get_clinev_data(biomarker_data)
                clinev_data = req.post_connection(ce_data,self.url_pattern,self.genome_version, tumor_type=tumor_type)
                clinev_data_json = json.loads(clinev_data)
                biomarker_data_json = ag.merge_dictionaries(biomarker_data, clinev_data_json)
                return biomarker_data_json
            else:
                request_data = {
                    "request_id": self.request_id,
                    "tumor_type": self.tumor_type
                }
                req.post_connection_params(request_data, self.url_pattern)
                cached_result = self.redis_client.get(self.request_id)
                result = json.loads(cached_result)
                # Get cached request from Redis client
                if cached_result:
                    biomarker_data = result["result"]
                    genome_version = result["genome_version"]

        except:
            print(traceback.format_exc())
            if self.error_logfile is not None:
                cur_dt = datetime.datetime.now()
                date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                print("error processing request: ", biomarker_data, file=self.error_logfile+str(date_time)+'.log')
            else:
                print(": error processing variant response: ;", traceback.format_exc())

        return biomarker_data
